import sys
from enum import Enum
from .key import Key


class PromptMode(Enum):
    disclose = "disclose"
    delete = "delete"


class Session:
    def __init__(self):
        self.keys = []
        self.unlock = False
        self.status = 127
        self.mode = None
        self.parse()

    def parse(self):
        for line in sys.stdin:
            parts = line.strip().partition(" ")
            cmd = parts[0]
            args = parts[2]
            if not hasattr(self, "_" + cmd):
                sys.stderr.write(f"Unknown command '{cmd}'\n")
                exit(127)
            more = getattr(self, "_" + cmd)(args)
            if more is False:
                break

    def _version(self, args):
        sys.stdout.write("version 0.0.0\n")
        sys.stdout.flush()

    def _key(self, args):
        self.keys.append(Key(args))

    def _unlock(self, args):
        self.unlock = True
        if len(self.keys) == 0:
            return False

    def _prompt(self, args):
        self.mode = PromptMode(args)
        return False

    def password(self, pw):
        """
        Sends a password to the daemon in response to an unlock command. Blocks
        until the daemon answers regarding the password's validity, returning
        True if it was correct and False otherwise.
        """
        sys.stdout.write(f"password {pw}\n")
        sys.stdout.flush()
        # TODO: Don't block
        line = sys.stdin.readline()
        if line == "password incorrect\n":
            return False
        elif line == "password correct\n":
            self.unlock = False
            self.parse()
            return True
        else:
            sys.stderr.write(f"Unexpected command '{line}'\n")
            exit(127)
